<?php

/**
 * Class ExpressionManufacturerTest
 *
 * Date: 26.03.13
 * Time: 12:30
 * @author Thomas Joußen <tjoussen@databay.de>
 */
class ExpressionManufacturerTest extends PHPUnit_Framework_TestCase
{

    /**
     * @var ReflectionClass
     */
    protected $reflection;

    /**
     * @var ExpressionManufacturer
     */
    protected $manufacturer;

    /**
     * @var string
     */
    protected $expected_pattern = '/%[0-9\.]+%|#-?[0-9\.]+#|\+[0-9]+\+|(Q\d+)(?=\=|<|>|\s|$)|Q[0-9]+\[[0-9]+\]|~.*?~|;[0-9]+:[0-9]+;|\$[0-9]+(?:,[0-9]+)*\$|\*[0-9]+(?:,[0-9]+)*\*/';

    public function setUp()
    {
        include_once __DIR__ . '/../../classes/Factory/ExpressionManufacturer.php';

        $this->reflection = new ReflectionClass("ExpressionManufacturer");
        $this->manufacturer = ExpressionManufacturer::_getInstance();
    }

    public function test_NotConstructable()
    {
        $this->assertFalse($this->reflection->isInstantiable());
        $method = $this->reflection->getMethod("__construct");
        $this->assertTrue($method->isPrivate());
    }

    public function test_NotCloneable()
    {
        $this->assertFalse($this->reflection->isCloneable());
        $method = $this->reflection->getMethod("__clone");
        $this->assertTrue($method->isPrivate());
    }

    public function test_Singleton()
    {
        $this->assertNotNull($this->manufacturer);
        $this->assertEquals($this->manufacturer, ExpressionManufacturer::_getInstance());
    }

    public function test_GetPattern()
    {
        $this->assertEquals($this->expected_pattern, $this->manufacturer->getPattern());
    }

    public function test_MatchPattern()
    {
        $subject = 'Q5 Q66 Q52[92] ~Hallo_Welt~ #5# +32+ %098% $3,4,5$ ;2:6;';
        $matches = $this->manufacturer->match($subject);
        $tokens = \explode(" ", $subject);
        foreach ($tokens as $token) {
            $this->assertContains($token, $matches);
        }
    }

    public function test_SupportedExpressions()
    {
        $expressions = array(
            "Q736" => 'AnswerOfQuestionExpression',
            "Q52[90]" => "ResultOfAnswerOfQuestionExpression",
            "%9%" => "PercentageResultExpression",
            "+14+" => "NumberOfResultExpression",
            "#2#" => "NumericResultExpression",
            "~Hello World~" => "StringResultExpression",
            ";2:5;" => "MatchingResultExpression",
            "$2,3,4,5$" => "OrderingResultExpression"
        );

        foreach ($expressions as $key => $expression) {
            include_once __DIR__ . "/../../classes/Expressions/" . $expression . ".php";
            $expression_object = $this->manufacturer->manufacture($key);
            $this->assertInstanceOf($expression, $expression_object);
        }
    }

    /**
     * @expectedException UnsupportedExpression
     */
    public function test_UnsupportedExpression()
    {
        $this->manufacturer->manufacture("%23#");
    }
}
